"""Integration tests for slice_objects covering sorting, integer, delta, calendar,
and mixed-bound slices."""

from datetime import datetime, timedelta, timezone
from types import SimpleNamespace

import pytest

from tals import slice_objects


def obj(dt: datetime) -> SimpleNamespace:
    return SimpleNamespace(start=dt)


def d(year, month, day, hour=0, tz=None):
    return datetime(year, month, day, hour, tzinfo=tz)


REF = datetime(2026, 3, 10)


# --- Sorting ---


def test_input_order_does_not_matter():
    a, b, c = obj(d(2026, 1, 1)), obj(d(2026, 2, 1)), obj(d(2026, 3, 1))
    assert slice_objects([c, a, b], ":", "start", REF) == [a, b, c]


def test_stable_sort_identical_timestamps():
    t = d(2026, 1, 1)
    x = SimpleNamespace(start=t, name="x")
    y = SimpleNamespace(start=t, name="y")
    result = slice_objects([x, y], ":", "start", REF)
    assert result == [x, y]  # original relative order preserved


# --- Single index ---


def test_index_first():
    a, b = obj(d(2026, 1, 1)), obj(d(2026, 2, 1))
    assert slice_objects([b, a], "0", "start", REF) is a


def test_index_last():
    a, b = obj(d(2026, 1, 1)), obj(d(2026, 2, 1))
    assert slice_objects([a, b], "-1", "start", REF) is b


def test_index_second_to_last():
    a, b, c = obj(d(2026, 1, 1)), obj(d(2026, 2, 1)), obj(d(2026, 3, 1))
    assert slice_objects([a, b, c], "-2", "start", REF) is b


# --- Integer slices ---


def test_slice_all():
    a, b, c = obj(d(2026, 1, 1)), obj(d(2026, 2, 1)), obj(d(2026, 3, 1))
    assert slice_objects([a, b, c], ":", "start", REF) == [a, b, c]


def test_slice_first_three():
    objs = [obj(d(2026, i, 1)) for i in range(1, 6)]
    assert slice_objects(objs, "0:3", "start", REF) == objs[:3]


def test_slice_all_but_last():
    a, b, c = obj(d(2026, 1, 1)), obj(d(2026, 2, 1)), obj(d(2026, 3, 1))
    assert slice_objects([a, b, c], ":-1", "start", REF) == [a, b]


def test_slice_last_three():
    objs = [obj(d(2026, i, 1)) for i in range(1, 6)]
    assert slice_objects(objs, "-3:", "start", REF) == objs[-3:]


def test_slice_all_but_first_and_last():
    a, b, c = obj(d(2026, 1, 1)), obj(d(2026, 2, 1)), obj(d(2026, 3, 1))
    assert slice_objects([a, b, c], "1:-1", "start", REF) == [b]


# --- Time-delta slices ---


def test_delta_open_end():
    # -7d from 2026-03-10 = 2026-03-03; only objects on/after that date
    a = obj(d(2026, 3, 2))  # before threshold
    b = obj(d(2026, 3, 3))  # exactly at threshold — included (start is inclusive)
    c = obj(d(2026, 3, 5))
    assert slice_objects([a, b, c], "-7d:", "start", REF) == [b, c]


def test_delta_open_start():
    # [:-7d] means timestamp < 2026-03-03
    a = obj(d(2026, 3, 2))  # included (< threshold)
    b = obj(d(2026, 3, 3))  # excluded (= threshold, end is exclusive)
    c = obj(d(2026, 3, 5))
    assert slice_objects([a, b, c], ":-7d", "start", REF) == [a]


def test_delta_both_sides():
    # -7d = Mar 3, -1d = Mar 9; [Mar 3, Mar 9)
    a = obj(d(2026, 3, 3))  # included
    b = obj(d(2026, 3, 8))  # included
    c = obj(d(2026, 3, 9))  # excluded (end exclusive)
    d_ = obj(d(2026, 3, 2))  # excluded (before start)
    assert slice_objects([a, b, c, d_], "-7d:-1d", "start", REF) == [a, b]


# --- Calendar slices ---


def test_calendar_this_month():
    a = obj(d(2026, 2, 28))  # before March
    b = obj(d(2026, 3, 1))  # start of March — inclusive
    c = obj(d(2026, 3, 10))
    assert slice_objects([a, b, c], "0M:", "start", REF) == [b, c]


def test_calendar_last_month_to_this():
    # [-1M:0M] = [Feb 1, Mar 1)
    a = obj(d(2026, 1, 31))  # excluded
    b = obj(d(2026, 2, 1))  # included (start inclusive)
    c = obj(d(2026, 2, 28))  # included
    e = obj(d(2026, 3, 1))  # excluded (end exclusive)
    assert slice_objects([a, b, c, e], "-1M:0M", "start", REF) == [b, c]


def test_calendar_this_year():
    a = obj(d(2025, 12, 31))  # excluded
    b = obj(d(2026, 1, 1))  # included
    c = obj(d(2026, 3, 10))
    assert slice_objects([a, b, c], "0Y:", "start", REF) == [b, c]


def test_calendar_this_week():
    # REF = 2026-03-10 (Tuesday), week_start=0 → week starts Mon Mar 9
    a = obj(d(2026, 3, 8))  # Sunday before — excluded
    b = obj(d(2026, 3, 9))  # Monday — included
    c = obj(d(2026, 3, 10))  # Tuesday — included
    assert slice_objects([a, b, c], "0W:", "start", REF, week_start=0) == [b, c]


def test_calendar_next_month_exclusive_end():
    # [0M:+1M] = [Mar 1, Apr 1)
    a = obj(d(2026, 2, 28))
    b = obj(d(2026, 3, 1))
    c = obj(d(2026, 3, 31))
    e = obj(d(2026, 4, 1))  # excluded
    assert slice_objects([a, b, c, e], "0M:+1M", "start", REF) == [b, c]


# --- Mixed bounds ---


def test_mixed_delta_int():
    # [-7d:-1]: filter >= Mar 3, then drop last
    a = obj(d(2026, 3, 3))
    b = obj(d(2026, 3, 7))
    c = obj(d(2026, 3, 10))
    # filtered = [a, b, c], [:-1] = [a, b]
    assert slice_objects([a, b, c], "-7d:-1", "start", REF) == [a, b]


def test_mixed_calendar_int():
    # [0M:-1]: filter >= Mar 1, then drop last
    a = obj(d(2026, 2, 15))
    b = obj(d(2026, 3, 1))
    c = obj(d(2026, 3, 10))
    # filtered = [b, c], [:-1] = [b]
    assert slice_objects([a, b, c], "0M:-1", "start", REF) == [b]



from datetime import datetime, timezone
from types import SimpleNamespace


def make_obj(dt: datetime) -> SimpleNamespace:
    return SimpleNamespace(start=dt)


def dt(year, month, day, hour=0, minute=0, tz=None):
    return datetime(year, month, day, hour, minute, tzinfo=tz)


REF = datetime(2026, 3, 10)
UTC = timezone.utc



from datetime import datetime, timezone
from types import SimpleNamespace

import pytest

from tals import slice_objects


def obj(dt: datetime) -> SimpleNamespace:
    return SimpleNamespace(start=dt)


REF = datetime(2026, 3, 10)
REF_AWARE = datetime(2026, 3, 10, tzinfo=timezone.utc)


# --- Empty list ---


def test_empty_single_index():
    assert slice_objects([], "0", "start", REF) is None


def test_empty_negative_index():
    assert slice_objects([], "-1", "start", REF) is None


def test_empty_slice():
    assert slice_objects([], ":", "start", REF) == []


def test_empty_delta_slice():
    assert slice_objects([], "-7d:", "start", REF) == []


def test_empty_calendar_slice():
    assert slice_objects([], "0M:", "start", REF) == []


# --- Out of bounds ---


def test_oob_positive_index():
    assert slice_objects([obj(REF)], "5", "start", REF) is None


def test_oob_negative_index():
    assert slice_objects([obj(REF)], "-5", "start", REF) is None


# --- Time range with no matches ---


def test_delta_no_matches():
    a = obj(datetime(2025, 1, 1))
    result = slice_objects([a], "0M:", "start", REF)
    assert result == []


# --- Timezone errors ---


def test_ref_naive_object_aware():
    a = obj(datetime(2026, 3, 1, tzinfo=timezone.utc))
    with pytest.raises(TypeError, match="awareness mismatch"):
        slice_objects([a], ":", "start", REF)


def test_ref_aware_object_naive():
    a = obj(datetime(2026, 3, 1))
    with pytest.raises(TypeError, match="awareness mismatch"):
        slice_objects([a], ":", "start", REF_AWARE)


def test_mixed_awareness_in_objects():
    a = obj(datetime(2026, 3, 1, tzinfo=timezone.utc))
    b = obj(datetime(2026, 3, 2))  # naive
    with pytest.raises(TypeError, match="awareness mismatch"):
        slice_objects([a, b], ":", "start", REF_AWARE)


def test_different_aware_timezones_allowed():
    utc_plus2 = timezone(
        datetime(2026, 1, 1).utcoffset() or __import__("datetime").timedelta(hours=2)
    )
    # Use two different UTC offsets
    from datetime import timedelta, timezone as tz

    tz1 = tz(timedelta(hours=0))
    tz2 = tz(timedelta(hours=2))
    a = obj(datetime(2026, 3, 1, tzinfo=tz1))
    b = obj(datetime(2026, 3, 2, tzinfo=tz2))
    result = slice_objects([a, b], ":", "start", REF_AWARE)
    assert len(result) == 2


# --- Missing attribute ---


def test_missing_attribute():
    bad = SimpleNamespace(other=REF)
    with pytest.raises(AttributeError):
        slice_objects([bad], ":", "other_field", REF)


def test_wrong_timestamp_key():
    a = SimpleNamespace(start=datetime(2026, 3, 1))
    with pytest.raises(AttributeError):
        slice_objects([a], ":", "created_at", REF)



import re
from dataclasses import dataclass


@dataclass
class IntBound:
    value: int


@dataclass
class DeltaBound:
    sign: int  # +1 or -1
    amount: int
    unit: str  # 'd', 'h', 'm'


@dataclass
class CalendarBound:
    offset: int  # 0 = current, -1 = previous, +1 = next
    period: str  # 'M', 'W', 'Y'


@dataclass
class IndexPosition:
    bound: IntBound


@dataclass
class SlicePosition:
    start: "IntBound | DeltaBound | CalendarBound | None"
    end: "IntBound | DeltaBound | CalendarBound | None"


_INT_RE = re.compile(r"^(-?\d+)$")
_DELTA_RE = re.compile(r"^([+-])(\d+)([dhm])$")
_CALENDAR_RE = re.compile(r"^([+-]?)(\d+)([MWY])$")


def _parse_bound(s: str) -> "IntBound | DeltaBound | CalendarBound":
    m = _INT_RE.match(s)
    if m:
        return IntBound(int(m.group(1)))

    m = _DELTA_RE.match(s)
    if m:
        sign = 1 if m.group(1) == "+" else -1
        return DeltaBound(sign, int(m.group(2)), m.group(3))

    m = _CALENDAR_RE.match(s)
    if m:
        sign_str, amount_str, period = m.group(1), m.group(2), m.group(3)
        amount = int(amount_str)
        offset = -amount if sign_str == "-" else amount
        return CalendarBound(offset, period)

    raise ValueError(f"Invalid bound expression: {s!r}")


def parse_position(position: str) -> "IndexPosition | SlicePosition":
    s = position.strip()
    if s.startswith("[") and s.endswith("]"):
        s = s[1:-1]

    if s.count(":") > 1:
        raise ValueError(f"Too many colons in position expression: {position!r}")

    if ":" not in s:
        m = _INT_RE.match(s)
        if m:
            return IndexPosition(IntBound(int(m.group(1))))
        if _DELTA_RE.match(s) or _CALENDAR_RE.match(s):
            raise ValueError(
                f"Calendar and time-delta expressions are not valid as a single index: {s!r}"
            )
        raise ValueError(f"Invalid position expression: {position!r}")

    left, right = s.split(":", 1)
    start = _parse_bound(left) if left else None
    end = _parse_bound(right) if right else None
    return SlicePosition(start, end)



from datetime import datetime, timedelta

from ._calendar import start_of_month, start_of_week, start_of_year
from ._parser import CalendarBound, DeltaBound

_UNIT_TO_KWARG = {"d": "days", "h": "hours", "m": "minutes"}


def resolve_to_datetime(
    bound: "DeltaBound | CalendarBound",
    reference_dt: datetime,
    week_start: int,
) -> datetime:
    if isinstance(bound, DeltaBound):
        delta = timedelta(**{_UNIT_TO_KWARG[bound.unit]: bound.amount})
        return reference_dt + bound.sign * delta

    if isinstance(bound, CalendarBound):
        if bound.period == "M":
            return start_of_month(reference_dt, bound.offset)
        if bound.period == "W":
            return start_of_week(reference_dt, bound.offset, week_start)
        if bound.period == "Y":
            return start_of_year(reference_dt, bound.offset)
        raise ValueError(f"Unknown calendar period: {bound.period!r}")

    raise TypeError(f"Expected DeltaBound or CalendarBound, got {type(bound)!r}")



"""Spec example table.

Objects:
    A: start = 2026-01-10
    B: start = 2026-02-05
    C: start = 2026-03-01
    D: start = 2026-03-10  ← reference_dt

reference_dt = 2026-03-10, timestamp_key = "start", week_start = 0

Note on [-60d:]: reference_dt - 60d = 2026-01-09. A (Jan 10) >= Jan 9, so A
is included. The spec example comment says [B, C, D] but that appears to be an
error — the threshold is Jan 9, not Jan 10.
"""

from datetime import datetime
from types import SimpleNamespace

import pytest

from tals import slice_objects


def obj(year, month, day, name):
    o = SimpleNamespace(start=datetime(year, month, day), name=name)
    return o


A = obj(2026, 1, 10, "A")
B = obj(2026, 2, 5, "B")
C = obj(2026, 3, 1, "C")
D = obj(2026, 3, 10, "D")

OBJECTS = [A, B, C, D]
REF = datetime(2026, 3, 10)


def sl(position):
    return slice_objects(OBJECTS, position, "start", REF, week_start=0)


@pytest.mark.parametrize(
    "position,expected",
    [
        ("[-1]", D),
        ("[-2]", C),
        ("[0]", A),
        ("[:]", [A, B, C, D]),
        ("[:-1]", [A, B, C]),
        ("[0M:]", [C, D]),
        ("[-1M:0M]", [B]),
        ("[-1M:]", [B, C, D]),
        ("[0Y:]", [A, B, C, D]),
        ("[0M:-1]", [C]),
        ("[:-2]", [A, B]),
    ],
)
def test_spec_example(position, expected):
    assert sl(position) == expected


def test_spec_example_60d():
    # reference_dt - 60d = 2026-01-09; A (Jan 10) >= Jan 9 → A is included
    assert sl("[-60d:]") == [A, B, C, D]



from datetime import datetime
from typing import Any

from ._parser import (
    CalendarBound,
    DeltaBound,
    IndexPosition,
    IntBound,
    SlicePosition,
    parse_position,
)
from ._resolver import resolve_to_datetime


def _is_temporal(bound) -> bool:
    return isinstance(bound, (DeltaBound, CalendarBound))


def _validate_tz(objects: list, reference_dt: datetime, timestamp_key: str) -> None:
    ref_aware = reference_dt.tzinfo is not None
    for i, obj in enumerate(objects):
        ts = getattr(obj, timestamp_key)  # AttributeError propagates naturally
        obj_aware = ts.tzinfo is not None
        if obj_aware != ref_aware:
            ref_state = "aware" if ref_aware else "naive"
            obj_state = "aware" if obj_aware else "naive"
            raise TypeError(
                f"Timezone awareness mismatch: reference_dt is {ref_state} but "
                f"object at index {i} has a {obj_state} timestamp."
            )


def slice_objects(
    objects: list[Any],
    position: str,
    timestamp_key: str,
    reference_dt: datetime,
    week_start: int = 0,
) -> "Any | list[Any]":
    _validate_tz(objects, reference_dt, timestamp_key)
    sorted_objs = sorted(objects, key=lambda o: getattr(o, timestamp_key))

    parsed = parse_position(position)

    if isinstance(parsed, IndexPosition):
        try:
            return sorted_objs[parsed.bound.value]
        except IndexError:
            return None

    assert isinstance(parsed, SlicePosition)
    start, end = parsed.start, parsed.end

    if not (_is_temporal(start) or _is_temporal(end)):
        s = start.value if isinstance(start, IntBound) else None
        e = end.value if isinstance(end, IntBound) else None
        return sorted_objs[s:e]

    # Temporal path: resolve datetime thresholds, filter, then apply integer bounds.
    start_dt = (
        resolve_to_datetime(start, reference_dt, week_start)
        if _is_temporal(start)
        else None
    )
    end_dt = (
        resolve_to_datetime(end, reference_dt, week_start)
        if _is_temporal(end)
        else None
    )

    # Inclusive start, exclusive end — consistent with Python slice semantics.
    filtered = [
        obj
        for obj in sorted_objs
        if (start_dt is None or getattr(obj, timestamp_key) >= start_dt)
        and (end_dt is None or getattr(obj, timestamp_key) < end_dt)
    ]

    s = start.value if isinstance(start, IntBound) else None
    e = end.value if isinstance(end, IntBound) else None
    return filtered[s:e]



import pytest
from tals._parser import (
    CalendarBound,
    DeltaBound,
    IndexPosition,
    IntBound,
    SlicePosition,
    parse_position,
)


# --- Single index ---


def test_index_positive():
    assert parse_position("0") == IndexPosition(IntBound(0))


def test_index_negative():
    assert parse_position("-1") == IndexPosition(IntBound(-1))


def test_index_large():
    assert parse_position("42") == IndexPosition(IntBound(42))


def test_index_with_brackets():
    assert parse_position("[-1]") == IndexPosition(IntBound(-1))


def test_index_calendar_raises():
    with pytest.raises(ValueError, match="not valid as a single index"):
        parse_position("0M")


def test_index_delta_raises():
    with pytest.raises(ValueError, match="not valid as a single index"):
        parse_position("-7d")


# --- Open slices ---


def test_slice_open_both():
    assert parse_position(":") == SlicePosition(None, None)


def test_slice_open_start():
    assert parse_position(":-1") == SlicePosition(None, IntBound(-1))


def test_slice_open_end():
    assert parse_position("-1:") == SlicePosition(IntBound(-1), None)


# --- Integer slices ---


def test_slice_int_int():
    assert parse_position("0:3") == SlicePosition(IntBound(0), IntBound(3))


def test_slice_neg_int():
    assert parse_position("-3:") == SlicePosition(IntBound(-3), None)


# --- Delta bounds ---


@pytest.mark.parametrize(
    "s,expected",
    [
        ("-7d", DeltaBound(-1, 7, "d")),
        ("+2h", DeltaBound(+1, 2, "h")),
        ("-30m", DeltaBound(-1, 30, "m")),
        ("+1d", DeltaBound(+1, 1, "d")),
    ],
)
def test_delta_bound(s, expected):
    result = parse_position(f"{s}:")
    assert result.start == expected


def test_delta_slice():
    assert parse_position("-7d:-1d") == SlicePosition(
        DeltaBound(-1, 7, "d"), DeltaBound(-1, 1, "d")
    )


# --- Calendar bounds ---


@pytest.mark.parametrize(
    "s,expected",
    [
        ("0M", CalendarBound(0, "M")),
        ("-1M", CalendarBound(-1, "M")),
        ("-2M", CalendarBound(-2, "M")),
        ("+1M", CalendarBound(1, "M")),
        ("0W", CalendarBound(0, "W")),
        ("-1W", CalendarBound(-1, "W")),
        ("0Y", CalendarBound(0, "Y")),
        ("-1Y", CalendarBound(-1, "Y")),
        ("+1Y", CalendarBound(1, "Y")),
    ],
)
def test_calendar_bound(s, expected):
    result = parse_position(f"{s}:")
    assert result.start == expected


def test_calendar_slice():
    assert parse_position("-1M:0M") == SlicePosition(
        CalendarBound(-1, "M"), CalendarBound(0, "M")
    )


# --- Mixed bounds ---


def test_mixed_delta_int():
    result = parse_position("-7d:-1")
    assert result == SlicePosition(DeltaBound(-1, 7, "d"), IntBound(-1))


def test_mixed_calendar_int():
    result = parse_position("0M:-1")
    assert result == SlicePosition(CalendarBound(0, "M"), IntBound(-1))


# --- Errors ---


def test_too_many_colons():
    with pytest.raises(ValueError):
        parse_position("1:2:3")


def test_invalid_bound():
    with pytest.raises(ValueError):
        parse_position("abc")


def test_invalid_unit():
    with pytest.raises(ValueError):
        parse_position("-7x:")



from datetime import datetime, timedelta

from dateutil.relativedelta import relativedelta


def start_of_month(dt: datetime, offset: int) -> datetime:
    first = dt.replace(day=1, hour=0, minute=0, second=0, microsecond=0)
    if offset == 0:
        return first
    return first + relativedelta(months=offset)


def start_of_week(dt: datetime, offset: int, week_start: int) -> datetime:
    midnight = dt.replace(hour=0, minute=0, second=0, microsecond=0)
    days_since_start = (dt.weekday() - week_start) % 7
    week_begin = midnight - timedelta(days=days_since_start)
    if offset == 0:
        return week_begin
    return week_begin + timedelta(weeks=offset)


def start_of_year(dt: datetime, offset: int) -> datetime:
    jan1 = dt.replace(month=1, day=1, hour=0, minute=0, second=0, microsecond=0)
    if offset == 0:
        return jan1
    return jan1.replace(year=jan1.year + offset)



from datetime import datetime, timedelta, timezone, timezone as tz

import pytest

from tals._parser import CalendarBound, DeltaBound
from tals._resolver import resolve_to_datetime

REF = datetime(2026, 3, 10, 12, 0)  # noon on a Tuesday
REF_AWARE = datetime(2026, 3, 10, 12, 0, tzinfo=timezone.utc)


# --- DeltaBound ---


def test_delta_days_negative():
    b = DeltaBound(-1, 7, "d")
    assert resolve_to_datetime(b, REF, 0) == REF - timedelta(days=7)


def test_delta_days_positive():
    b = DeltaBound(+1, 3, "d")
    assert resolve_to_datetime(b, REF, 0) == REF + timedelta(days=3)


def test_delta_hours():
    b = DeltaBound(-1, 2, "h")
    assert resolve_to_datetime(b, REF, 0) == REF - timedelta(hours=2)


def test_delta_minutes():
    b = DeltaBound(+1, 30, "m")
    assert resolve_to_datetime(b, REF, 0) == REF + timedelta(minutes=30)


def test_delta_preserves_aware_tz():
    b = DeltaBound(-1, 1, "d")
    result = resolve_to_datetime(b, REF_AWARE, 0)
    assert result.tzinfo == timezone.utc


# --- CalendarBound: Month ---


def test_month_current():
    b = CalendarBound(0, "M")
    assert resolve_to_datetime(b, REF, 0) == datetime(2026, 3, 1, 0, 0)


def test_month_previous():
    b = CalendarBound(-1, "M")
    assert resolve_to_datetime(b, REF, 0) == datetime(2026, 2, 1, 0, 0)


def test_month_two_back():
    b = CalendarBound(-2, "M")
    assert resolve_to_datetime(b, REF, 0) == datetime(2026, 1, 1, 0, 0)


def test_month_next():
    b = CalendarBound(1, "M")
    assert resolve_to_datetime(b, REF, 0) == datetime(2026, 4, 1, 0, 0)


def test_month_year_rollback():
    # -2M from February → December of previous year
    ref = datetime(2026, 2, 15)
    b = CalendarBound(-2, "M")
    assert resolve_to_datetime(b, ref, 0) == datetime(2025, 12, 1, 0, 0)


def test_month_preserves_aware_tz():
    b = CalendarBound(0, "M")
    result = resolve_to_datetime(b, REF_AWARE, 0)
    assert result == datetime(2026, 3, 1, 0, 0, tzinfo=timezone.utc)
    assert result.tzinfo == timezone.utc


# --- CalendarBound: Week ---


def test_week_current_monday_start():
    # REF = 2026-03-10, which is a Tuesday; week_start=0 (Monday)
    # Monday of that week = 2026-03-09
    b = CalendarBound(0, "W")
    assert resolve_to_datetime(b, REF, 0) == datetime(2026, 3, 9, 0, 0)


def test_week_current_sunday_start():
    # REF = 2026-03-10 (Tuesday); week_start=6 (Sunday)
    # Sunday of that week = 2026-03-08
    b = CalendarBound(0, "W")
    assert resolve_to_datetime(b, REF, 6) == datetime(2026, 3, 8, 0, 0)


def test_week_previous():
    b = CalendarBound(-1, "W")
    assert resolve_to_datetime(b, REF, 0) == datetime(2026, 3, 2, 0, 0)


def test_week_current_on_start_day():
    # reference_dt is Monday itself (week_start=0)
    ref = datetime(2026, 3, 9)  # Monday
    b = CalendarBound(0, "W")
    assert resolve_to_datetime(b, ref, 0) == datetime(2026, 3, 9, 0, 0)


# --- CalendarBound: Year ---


def test_year_current():
    b = CalendarBound(0, "Y")
    assert resolve_to_datetime(b, REF, 0) == datetime(2026, 1, 1, 0, 0)


def test_year_previous():
    b = CalendarBound(-1, "Y")
    assert resolve_to_datetime(b, REF, 0) == datetime(2025, 1, 1, 0, 0)


def test_year_next():
    b = CalendarBound(1, "Y")
    assert resolve_to_datetime(b, REF, 0) == datetime(2027, 1, 1, 0, 0)
